'use strict';

/**
 * 构造方法
 * @param {*} options: {
 *      container: 父容器
 *      direction: 方向，Direction.Horizontal, Direction.Vertical
 *      pageSize: Horizontal 为页面宽度，Vertical 为页面高度
 *      lineHeight: 行的末尾可能被裁剪, 上下滑动时保留上一页最后一行
 *      pageChange: function(int currPage)
 */
var Swipper = function (options) {
    this.container = options.container;
    this.direction = options.direction;
    this.pageSize = options.pageSize;
    this.lineHeight = options.lineHeight || 0;
    this.pageChange = options.pageChange || function() {};
    this.scrollChange = options.scrollChange || function() {};

    this.curPage = 0;
    if (this.isHorizontal()) {
        this.pageCount = Math.ceil(this.container.scrollWidth / this.pageSize);
    } else {
        this.pageCount = Math.ceil(this.container.scrollWidth / this.pageSize);
    }

    this.init();
};

Swipper.prototype.isHorizontal = function () {
    return this.direction == Direction.HORIZONTAL;
};

Swipper.prototype.init = function () {
    this.initParams();
    this.addEvents();
};

Swipper.prototype.initParams = function () {
    var param = {
        touchStartTime: 0, 
        lastMoveTime: 0,
        startTouch: null, 
        lastTouch: null,
        isMoving: false,
        state: this.StateEnum.IDLE,

        currPage: 0,

        TAP_TIMEOUT: 100,
        DOUBLE_TAP_TIMEOUT: 300,
        DOUBLE_TAP_MIN_TIME: 40,
        FLING_TIMEOUT: 300,

        PROCESS_INTERVAL: 100,
        PROCESS_SLOP: 10, 
    };
    jQuery.extend(this, this, param);
};

Swipper.prototype.handleTouchStart = function (e) {
    console.log("Test handleTouchStart ");
    var touch = e.touches[0];
    this.touchStartTime = this.getCurrTime();
    this.isMoving = false;
    this.startTouch = touch;
    this.lastTouch = touch;
    this.state = this.StateEnum.READY;
};

Swipper.prototype.handleTouchMove = function (e) {
    console.log("Test handleTouchMove ");
    e.preventDefault();
    this.isMoving = true;
    this.lastMoveTime = this.getCurrTime();
    var touch = e.touches[0];

    if (this.isHorizontal()) {
        var deltaX = touch.screenX - this.lastTouch.screenX;
        this.moveLeft(-deltaX);
    } else {
        var deltaY = touch.screenY - this.lastTouch.screenY;
        this.moveTop(-deltaY);
    }
    this.lastTouch = touch;

    this.scrollChange(this.isScrollToLeft(), this.isScrollToRight(), this.isScrollToTop(), this.isScrollToBottom());
};

Swipper.prototype.handleTouchEnd = function (e) {
    console.log("Test handleTouchEnd ");
    if (!this.isMoving) return;

    e.preventDefault();
    this.scrollChange(this.isScrollToLeft(), this.isScrollToRight(), this.isScrollToTop(), this.isScrollToBottom());
    this.moveLastTime = this.getCurrTime();

    var delta = 0;
    if (this.isHorizontal()) {
        delta = this.lastTouch.screenX - this.startTouch.screenX;
    } else {
        delta = this.lastTouch.screenY - this.startTouch.screenY;
    }

    console.log("handleTouchEnd:" + delta);
    if (this.getCurrTime() - this.touchStartTime < this.FLING_TIMEOUT) {
        if (delta < 0) {
            this.nextPage();
        } else {
            this.prevPage();
        }
        return;
    }

    var FRICATION = this.pageSize / 3;
    if (Math.abs(delta) < FRICATION) {
        this.gotoPage(this.currPage);
        return;
    }
    if (delta < 0) {
        this.nextPage();
    } else {
        this.prevPage();
    }
};

Swipper.prototype.handleScroll = function(e) {
    console.log("Test handleScroll isScrollToRight:" + this.isScrollToRight());
    this.scrollChange(this.isScrollToLeft(), this.isScrollToRight(), this.isScrollToTop(), this.isScrollToBottom());
}

Swipper.prototype.isScrollToLeft = function() {
    return $(this.container).scrollLeft() < 1;
}

Swipper.prototype.isScrollToRight = function() {
    var c = this.container;
    var width = c.scrollWidth + this.toInt(c.style.marginLeft) + this.toInt(c.style.marginRight) +
                    this.toInt(c.style.paddingLeft) + this.toInt(c.style.paddingRight);
    return (width - c.scrollLeft - this.pageSize) < 1;
}

Swipper.prototype.isScrollToTop = function() {
    return $(this.container).scrollTop() < 1;
}

Swipper.prototype.isScrollToBottom = function() {
    var c = this.container;
    var height = c.scrollWidth + this.toInt(c.style.marginTop) + this.toInt(c.style.marginBottom) +
                    this.toInt(c.style.paddingTop) + this.toInt(c.style.paddingBottom);
    return (height - c.scrollToTop - this.pageSize) < 1;
}

Swipper.prototype.nextPage = function() {
    console.log("Test nextPage");

    this.currPage++;
    if (this.currPage >= this.pageCount) this.currPage = this.pageCount - 1;
    this.gotoPage(this.currPage, true);
}

Swipper.prototype.prevPage = function() {
    this.currPage--;
    if (this.currPage < 0) this.currPage = 0;
    this.gotoPage(this.currPage, true);
}

Swipper.prototype.getCurrTime = function() {
    return (new Date()).getTime();
}

Swipper.prototype.StateEnum = {
    IDLE: 0,
    READY: 1,
    MOVE: 2,
}

Swipper.prototype.getCurrPage = function () {
    if (this.isHorizontal()) {
        return this.getPageIndex($(this.container).scrollLeft());
    } else {
        return this.getPageIndex($(this.container).scrollTop());
    }
}

Swipper.prototype.getPageIndex = function(offset) {
    return Math.floor(offset * 1.0 / this.pageSize);
}

Swipper.prototype.moveLeft = function(left, animation) {
    var left = $(this.container).scrollLeft() + left;
    this.scrollToLeft(left, false);
}

Swipper.prototype.scrollToLeft = function(left, animation) {
    if (animation) {
        $(this.container).animate({
            "scrollLeft": left
        });
    } else {
        $(this.container).scrollLeft(left);
    }
}

Swipper.prototype.moveTop = function(distance, animation) {
    var top = $(this.container).scrollTop() + distance;
    this.scrollToTop(top, false);
}

Swipper.prototype.scrollToTop = function(distance, animation) {
    if (animation) {
        $(this.container).animate({
            "scrollTop": distance
        });
    } else {
        $(this.container).scrollTop(distance);
    }
}

Swipper.prototype.gotoPage = function (pageIndex, animation) {
    console.log("Test gotoPage:" + pageIndex);
    this.currPage = pageIndex;

    if (this.isHorizontal()) {
        var left = pageIndex * this.pageSize;

        console.log("Test gotoPage scrollToLeft:" + left);
        this.scrollToLeft(left, animation);
    } else {
        console.log("Test gotoPage pageSize:" + this.pageSize);
        var top = pageIndex * this.pageSize;
        //TODO improve code, 因为页面最低部一页可能被裁剪，翻到下一页时，保留上页底部
        top -= this.lineHeight * 0.5;
        if (top < 0) top = 0;

        console.log("Test gotoPage scrollToTop:" + top);
        this.scrollToTop(top, animation);
    }

    if (this.pageChange != null) this.pageChange(pageIndex);
};

Swipper.prototype.toInt = function(str) {
    if (!str) return 0;
    var ret = parseInt(str);
    if (isNaN(ret)) return 0;
    return ret;
}

// 添加事件绑定
Swipper.prototype.addEvents = function () {
    this.container.addEventListener('touchstart', this.handleTouchStart.bind(this), {passive: false});
    this.container.addEventListener('touchmove', this.handleTouchMove.bind(this), {passive: false});
    this.container.addEventListener('touchend', this.handleTouchEnd.bind(this), {passive: false});
    this.container.addEventListener('scroll', this.handleScroll.bind(this), {passive: false});
};